# (c) I. Marti-Vidal (Univ. Valencia) 2020.

# Script to solve for the instrumental polarization of (synthetic)
# EHT observations of a resolved calibrator, using the "polarization
# self-calibration" strategy (described in Cotton 1993).
# This script complements the contents of the paper:

# I. Marti-Vidal et al. (2020), A&A, 646, A52



import numpy as np
import pylab as pl
import os,sys
import pickle as pk

from task_polsolve import polsolve as polsolve

# Number of self-cal iterations:
NITER = 10

# Weighting and robust:
weighting = 'briggs'
robust = -2.0

# Name of measurment set:
MSNAME = 'POLSIM_CASE-3.ms'

# First, we re-fill the "corrected" column of the measurement set 
# with the original data (to undo any previous calibration):
clearcal(vis=MSNAME,addmodel=True)


# FIRST CLEAN (INTERACTIVE MODE):
if True:
 os.system('rm -rf %s_clean*'%MSNAME[:-3])
 tclean(vis=MSNAME,
    imagename='%s_clean'%MSNAME[:-3],
    specmode = 'mfs',
    niter = 200,
    interactive=True,
    imsize=1024,
    cell = '0.000001arcsec',
    stokes = 'IQUV',
    weighting = weighting,
    robust=robust,
    deconvolver='hogbom',
    gain=0.1,
    restart=False)

 os.system('rm -rf %s_TMASK.mask'%MSNAME[:-3])
 os.system('cp -r %s_clean.mask %s_TMASK.mask'%(MSNAME[:-3],MSNAME[:-3]))


# ITERATE!!
for i in range(NITER):

  print('\n\n ITERATION %i\n\n'%i)

## We CLEAN the image, using the visibilities stored in 
# the "corrected" column of the measurement set:
  os.system('rm -rf %s_clean*'%MSNAME[:-3])
  tclean(vis=MSNAME,
    imagename='%s_clean'%MSNAME[:-3],
    specmode = 'mfs',
    niter = 200,
    interactive=False,
    imsize=1024,
    cell = '0.000001arcsec',
    stokes = 'IQUV',
    mask = '%s_TMASK.mask'%MSNAME[:-3], ## We use the mask that we created manually.
    weighting = weighting, robust=robust,
    deconvolver='hogbom',
    gain=0.1,
    restart=False)

# We remove the previous Dterm table:
  os.system('rm -rf %s.spw_0.Dterms'%MSNAME)


# Solve for the Dterms:
  polsolve(vis=MSNAME,
## If we set "target_field", the Dterms will not only be 
# estimated, but also APPLIED to the "corrected" column.
# This is ESSENTIAL for the self-cal approach to work: 
         target_field='POLSIM', # Source name
# Fit all the Dterms:
         DRSolve = [True for i in range(8)],
         DLSolve = [True for i in range(8)],
## Antenna mounts (the ordering should be that of the ANTENNA table):
         mounts=['AZ', 'NR', 'NR', 'AZ', 'NL', 'NL', 'AZ', 'NL'],
## When "CLEAN_models" is a full-polarization CASA image,
## the self-calibration approach is activated:
         CLEAN_models='%s_clean.model'%MSNAME[:-3],
# These values are not really used:
         frac_pol=[0.], EVPA=[0.], PolSolve=[False],
## Set this to True, if you want to test the linear Dterm model:
         linear_approx=False)

## Save the Dterm table with a name that contains the iteration number:
  os.system('rm -rf ITER_%i.Dterms'%i)
  os.system('cp -r %s.spw_0.Dterms ITER_%i.Dterms'%(MSNAME,i))


## THAT'S IT! After the last iteration, the final data should se stored 
# in the "corrected" column of the measurement set.



## READ FITTED DTERMS:
tb.open('ITER_%i.Dterms'%i)
DTS = tb.getcol('CPARAM')
tb.close()



## RECOVER SIMULATED DTERMS:
INFILE = open('%s_Dterms.dat'%MSNAME[:-3],'rb')
DRs,DLs = pk.load(INFILE) ; INFILE.close()




#### PLOT TRUE vs. FITTED DTERM COMPARISON:
fig = pl.figure()
sub = fig.add_subplot(111)
for ant in range(8):
  sub.plot(DRs.real,DTS[0,0,:].real,'or')
  sub.plot(DRs.imag,DTS[0,0,:].imag,'sr')
  sub.plot(DLs.real,DTS[1,0,:].real,'ob')
  sub.plot(DLs.imag,DTS[1,0,:].imag,'sb')

sub.plot([],[],'or',label='DR Real')
sub.plot([],[],'sr',label='DR Imag')
sub.plot([],[],'ob',label='DL Real')
sub.plot([],[],'sb',label='DL Imag')
sub.plot(np.array([-0.5,0.5]),np.array([-0.5,0.5]),':k')
pl.legend(numpoints=1,loc=4)
pl.xlabel('True Dterms')
pl.ylabel('PolSolve Dterms')
pl.suptitle('Pol. SelfCal')
pl.show()










