# (c) I. Marti-Vidal (Univ. Valencia) 2020.

# Script to solve for the instrumental polarization of (synthetic)
# EHT observations of a resolved calibrator, using the "source slicing".
# strategy, as originally implemented in LPCAL (Leppanen 1995).
# This script complements the contents of the paper:

# I. Marti-Vidal et al. (2020), A&A, 646, A52





import numpy as np
import pylab as pl
import os, sys
import glob
import pickle as pk
from task_polsolve import polsolve as polsolve

# Weighting and robust:
weighting = 'briggs'
robust = -2.0

# Name of measurment set:
MSNAME = 'POLSIM_CASE-5.ms'


# CLEAN (INTERACTIVE MODE):
os.system('rm -rf %s_clean*'%MSNAME[:-3])
tclean(vis=MSNAME,
    imagename='%s_clean'%MSNAME[:-3],
    specmode = 'mfs',
    niter = 200,
    interactive=True,
    imsize=1024,
    cell = '0.000001arcsec',
    stokes = 'IQUV',
    weighting = weighting,
    robust=robust,
    deconvolver='hogbom',
    gain=0.1,
    restart=False)


# The user should use the task "imview" to get the pixel coordinates of the
# BLC and TRC of the boxes that will define the polarization sub-components.
# In this case, we are using several sub-components, stored in the "MyRegions.rg" 
# file. The helper task called "CCextract" takes all the CASA (or DS9) regions 
# defined there (you can use  boxes, circles, ellipses, etc.), extracts the 
# CC components of each region and saves them in ascii format, to be read by 
# PolSolve:
CCextract(model_image='%s_clean.model'%MSNAME[:-3],
          regions='manual',make_plot=True)

## Get all the subcomponent files:
ALLCCs = glob.glob('%s_clean.model.CC??'%MSNAME[:-3])

# NOTE: CCextract has an option to make a plot of the CLEAN deltas in each
# sub-component (using different colors). In addition, you should pay attention
# to the warnings regarding CLEAN components appearing in more than one region.
# If this happens, the CLEAN component is only added to the region first 
# appearing in the list. This behaviour can indeed be used to make fancy 
# boolean region conditions!


# Solve for Dterms and source polarization:
polsolve(vis = MSNAME,
## The mounts should be given following the order of the ANTENNA table:
         mounts=['AZ', 'NR', 'NR', 'AZ', 'NL', 'NL', 'AZ', 'NL'],
## This is the list of ascii files created by "CCextract":
         CLEAN_models=ALLCCs,
## Initial guesses of the polarization state of each sub-component:
         frac_pol=[0. for pi in ALLCCs], EVPA=[0. for pi in ALLCCs], 
## Solve for the polarization of all three sub-components:
         PolSolve=[True for pi in ALLCCs],
## Apply the full non-linear Dterm model:
         linear_approx=False)

## Change the name of the Dterm CASA table, to keep it:
os.system('rm -rf PolSolve_Similarity.Dterms')
os.system('cp -r %s.spw_0.Dterms PolSolve_Similarity.Dterms'%MSNAME)



## READ FITTED DTERMS:
tb.open('%s.spw_0.Dterms'%MSNAME)
DTS = tb.getcol('CPARAM')
tb.close()



## RECOVER SIMULATED DTERMS:

INFILE = open('%s_Dterms.dat'%MSNAME[:-3],'rb')
DRs,DLs = pk.load(INFILE) ; INFILE.close()




#### PLOT CORRELATION OF DTERMS:
fig = pl.figure()
sub = fig.add_subplot(111)
for ant in range(8):
  sub.plot(DRs.real,DTS[0,0,:].real,'or')
  sub.plot(DRs.imag,DTS[0,0,:].imag,'sr')
  sub.plot(DLs.real,DTS[1,0,:].real,'ob')
  sub.plot(DLs.imag,DTS[1,0,:].imag,'sb')

sub.plot([],[],'or',label='DR Real')
sub.plot([],[],'sr',label='DR Imag')
sub.plot([],[],'ob',label='DL Real')
sub.plot([],[],'sb',label='DL Imag')
sub.plot(np.array([-0.5,0.5]),np.array([-0.5,0.5]),':k')
pl.legend(numpoints=1,loc=4)
pl.xlabel('True Dterms')
pl.ylabel('PolSolve Dterms')
pl.suptitle('SubComponent Fitting')
pl.show()










